/*++

Copyright (c) Sydus.

Module:

    Sha1.h

Authors:

    Serge Zloto (szloto) 2007-01-11

--*/

#pragma once

#include "base.h"
#include <string>

/**
 * A SHA-1 hasher that can stream data in (operator <<).
 * The amount of data it can handle is system dependent: it is 2^32 on 32 bit
 * platforms, and 2^64 on 64 bit platforms.
 */
class Sha1
{
	public:
		typedef BString::size_type size_type;
		typedef BString::value_type value_type;

		static const uint32 KHashSize = 20;
		static const uint32 KBlockSize = 64;

		Sha1();
		~Sha1();

		BString Hash(const BString & data);
		BString Hash(const std::string & data);

 		BString Hash();
 		Sha1 & operator << (const BString & data);
		Sha1 & operator << (const std::string & data);

		Sha1 & Input(const value_type * data, size_type dataSize);

	private:
		/**
		 * The input is broken into 16 32-bit chunks which
		 * are extended into 80 32-bit chunks
		 */
		static const uint32 KChunkSize32 = 16;
		static const uint32 KExtendedChunkSize32 = 80;
		static const uint32 KChunkSize8 = KChunkSize32 * 4;

		/**
		 * ProcessChunk assumes we have a complete chunk in iChunk
		 */
		void ProcessChunk();
		void ConvertChunkToHostEndian();
		void ExtendChunkFrom16To80Values();
		void CalculateChunkHash();
		void ResetState();

		size_type iInputSize;
		uint32 iChunkOffset;
		
		struct Result
		{ 
			uint32 h0;
			uint32 h1;
			uint32 h2;
			uint32 h3;
			uint32 h4;
		} iState;

		union
		{
			uint32 iChunk32[KExtendedChunkSize32];
			value_type iChunk8[KChunkSize8];
		};
};
