/*++

Copyright (c) Sydus.

Module:

    quality.h

Authors:

    Sharvil Nanavati (sharvil) 2005-11-05

--*/

#pragma once

#include <memory>
#include <climits>

//
// Macro concatenation with full preprocessor expansion. The MACRO_CONCATENATE
// macro is the one to use.
//
#define MACRO_CONCATENATE(arg1, arg2) PRIVATE_MACRO_CONCATENATE(arg1, arg2)
#define PRIVATE_MACRO_CONCATENATE(arg1, arg2) arg1##arg2

//
// Macro stringizing with full preprocessor expansion. The MACRO_STRINGIZE
// macro is the one to use.
//
#define MACRO_STRINGIZE(arg) PRIVATE_MACRO_STRINGIZE(arg)
#define PRIVATE_MACRO_STRINGIZE #x

//
// This macro should be used whenever a parameter is passed into a function is
// not used. This prevents compiler warnings and makes the intent clear. Note
// that expressions with side effects should not be used with this macro.
//
#define UNUSED_PARAMETER(Param) ((void)(Param))

//
// This macro is to be used when a variable is declared but not used
// (intentionally). It should be used sparingly.
//
#define UNUSED_VARIABLE(var) ((void)(var))

//
// Compile time assertions - use C_ASSERT macro.
//
template <bool> struct CompileTimeAssert;
template <> struct CompileTimeAssert <true> {};
template <int> struct CompileTest {};
#define C_ASSERT(expression) typedef CompileTest <sizeof(CompileTimeAssert <(expression)>)> MACRO_CONCATENATE(CompileTime_, __LINE__)

//
// Define some sized integral types and make sure they actually represent the
// size they claim to.
//
typedef const char *       const_cstring;
typedef char *             cstring;

//
// We must use #define since a typedef of int -> int32 causes problems in the
// compiler. ISO C++ states that 'main' must return 'int' but the compilers
// do not resolve int32 as int before generating the warning so we must do it
// in the preprocessor instead. The rest are there for symmetry. These defines
// are safe since they are for simple, non-pointer, non-decorated types. These
// defines must be used until the compilers are fixed (read: forever).
//
#define uint8      unsigned char
#define int8       char
#define uint16     unsigned short
#define int16      short
#define uint32     unsigned int
#define int32      int
#define uint64     unsigned long long
#define int64      long long
#define nativeint  int
#define nativeuint unsigned int

C_ASSERT(CHAR_BIT == 8);
C_ASSERT(sizeof(uint8)      == 1);
C_ASSERT(sizeof(int8)       == 1);
C_ASSERT(sizeof(uint16)     == 2);
C_ASSERT(sizeof(int16)      == 2);
C_ASSERT(sizeof(uint32)     == 4);
C_ASSERT(sizeof(int32)      == 4);
C_ASSERT(sizeof(uint64)     == 8);
C_ASSERT(sizeof(int64)      == 8);
C_ASSERT(sizeof(nativeint)  >= 4);
C_ASSERT(sizeof(nativeuint) >= 4);

#define NATIVEINT_BITS   (sizeof(nativeint) * 8)
#define NATIVEUINT_BITS  (sizeof(nativeuint) * 8)
#define NATIVEINT_BYTES  (sizeof(nativeint))
#define NATIVEUINT_BYTES (sizeof(nativeuint))
#define NATIVEINT_MAX    (INT_MAX)
#define NATIVEUINT_MAX   (UINT_MAX)
#define NATIVEINT_MIN    (INT_MIN)
#define NATIVEUINT_MIN   (0)

//
// The NonCopyable provides a simple method to create classes that should not be
// copied (e.g. resource classes like sockets or mutexes). By privately
// inheriting from this class, we can prevent derived classes from being copied.
//
class NonCopyable
{
	protected:
		NonCopyable() {}
		~NonCopyable() {}

	private:
		NonCopyable(const NonCopyable &);
		NonCopyable & operator = (const NonCopyable &);
};

//
// The NonInstantiable class provides a method to create a class that cannot be
// instantiated (e.g. a class with just static members). By inheriting from this
// class, we can prevent derived classes from being instantiated. Note that if
// class C inherits from NonInstantiable, class D : public C cannot be
// instantiated either.
//
class NonInstantiable
{
	private:
		NonInstantiable() {}
};

//
// Inheriting from this class ensures that instances are not the result of
// the new operator, that is, an instance cannot exist on the heap by itself.
// Instances can be allocated on the stack (auto variables) or be aggregated
// within another object.
//
// This way, intances of NonPointer do not have an independent lifetime,
// thus are easier to track, and less likely to participate in resource leakage.
//
class NonPointer
{
	private:
		static void * operator new(size_t bytes);
		static void operator delete(void * ptr);
};
